import com.midori.cloud.renderer.jira.model.component.StatusCategory

filename = "${title} - ${new Date().format('yyyy-MM-dd')}.xlsx".toString() // "filename" is a standard Java String, not a GString

// check dependency
if (!binding.variables.containsKey('fieldHelper')) {
	throw new Exception("The script \"software-report.groovy\" requires the prior execution of \"field-helper-tool.groovy\".\nUse this expression in one of the topmost cells: <mt:execute script=\"field-helper-tool.groovy\"/><mt:execute script=\"software-report.groovy\"/>")
}

softwareReport = new SoftwareReportTool(fieldHelper: fieldHelper)

class SoftwareReportTool {
	def fieldHelper

	def getAssignee(issue) {
		issue.assignee?.displayName ?: '(Unassigned)'
	}

	def getComponents(issue) {
		issue.components ? issue.components.collect{ it.name }.join(', ') : '(No component)'
	}

	def getFixVersions(issue) {
		issue.fixVersions ? issue.fixVersions.collect{ it.name }.join(', ') : '(No version)'
	}

	def getEpic(issue) {
		fieldHelper.getFieldValueByName(issue, 'Epic Link') ?: '(No epic)'
	}

	def getSprint(issue) {
		fieldHelper.getFieldValueByName(issue, 'Sprint') ?: '(No sprint)'
	}

	def getStoryPoints(issue) {
		fieldHelper.getFieldValueByName(issue, 'Story Points') ?: 0
	}

	def getCommittedStoryPoints(issue) {
		getStoryPoints(issue)
	}

	/**
	 * Returns the value in the "Story Points" field if the issue is in a status with category "Done", 0 otherwise.
	 */
	def getCompletedStoryPoints(issue) {
		(issue.status.statusCategory.type == StatusCategory.StatusCategoryType.DONE) ? getCommittedStoryPoints(issue) : 0
	}
}
