import com.google.common.collect.LinkedListMultimap

// check dependency
if (!binding.variables.containsKey('tempo')) {
	throw new Exception("The script \"tempo-report-tool.groovy\" requires the prior execution of \"tempo-tool.groovy\".\nUse this expression in one of the topmost cells: <mt:execute script=\"tempo-tool.groovy\"/><mt:execute script=\"tempo-report-tool.groovy\"/>")
}

tempoReport = new tempoReportTool(tempo: tempo)

class tempoReportTool {
	def tempo

	def getReporter(issue) {
		issue.reporter?.displayName ?: '(Anonymous)'
	}

	def getAssignee(issue) {
		issue.assignee?.displayName ?: '(Unassigned)'
	}

	def getAffectedVersions(issue) {
		issue.affectedVersions ? issue.affectedVersions.collect{ it.name }.join(', ') : '(No version)'
	}

	def getFixVersions(issue) {
		issue.fixVersions ? issue.fixVersions.collect{ it.name }.join(', ') : '(No version)'
	}

	def getComponents(issue) {
		issue.components ? issue.components.collect{ it.name }.join(', ') : '(No component)'
	}

	def getAccount(issue, worklog) {
		isValidWorklog(worklog) ? (tempo.getAccount(issue, worklog) ?: '(No account)') : null
	}

	def getTeam(issue, worklog) {
		isValidWorklog(worklog) ? (tempo.getTeam(issue) ?: '(No team)') : null
	}

	def getWorklogDate(worklog) {
		isValidWorklog(worklog) ? worklog.startDate : null
	}

	/**
	 * Returns the worklog author's display name for a valid worklog, or the description with the error message for a dummy one.
	 */
	def getWorklogUser(worklog) {
		isValidWorklog(worklog) ? worklog.author.displayName : worklog.description
	}

	def getWorklogAttributes(worklog) {
		if(isValidWorklog(worklog)) {
			def workAttributes = tempo.getWorkAttributes(worklog)
			return workAttributes ?: '(No worklog attributes)'
		} else {
			return null
		}
	}

	def getWorkedHours(worklog) {
		isValidWorklog(worklog) ? worklog.timeSpentSeconds/3600 : null
	}

	def getBillableHours(worklog) {
		isValidWorklog(worklog) ? worklog.billableSeconds/3600 : null
	}

	/**
	 * Returns whether the passed worklog is a valid one returned by Tempo
	 * (not a dummy one carrying a Tempo connection error message, for example).
	 */
	private static isValidWorklog(worklog) {
		(worklog.startDate != null)
	}

	def getRepeatingTempoWorklogs(issues) {
		def result = LinkedListMultimap.create() // keeps insertion order for both keys and values

		issues.each {
			it ->
				def tempoWorklogs = tempo.getWorklogs(it.key) ?: [[:]]
				result.putAll(it, tempoWorklogs)
		}
		return result.entries()
	}
}